/**
 * @file pboeservo.c
 * This is the main program start point.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/thread.h>
#include <propeller.h>

#include "propboe.h"

/* stacks for COGs */
static int PingStack[STACK_SIZE];

/* per-thread library variables */
static _thread_state_t PingThreadData;

/* variables that we share between cogs */

/* Ping))) COG variables */
static volatile int PingTime;

/*
 * Ping))) COG function
 */
void PingThread(void *arg __attribute__((unused)))
{
    int t0 = 0;
    int t1 = 0;
    int mstime = (CLKFREQ/1000);
    int ustime = (CLKFREQ/1000000);
    int pinmask = (1<<PING_PIN);

    for(;;) {
        
        OUTA |=  pinmask;
        DIRA |=  pinmask;
        waitcnt(5*mstime+CNT);      // trigger time
        OUTA &= ~pinmask;
        DIRA &= ~pinmask;

        waitpeq(pinmask,pinmask);   // wait for rising edge
        t0 = CNT;
        waitpne(pinmask,pinmask);   // wait for falling edge
        t1 = CNT;

        PingTime = ((t1-t0)/ustime)>>1;

        waitcnt(5*mstime+CNT);
    }
}

/* start the threads */

void startPingThread(void)
{
    int n = _start_cog_thread(PingStack + STACK_SIZE, PingThread, NULL, &PingThreadData);
    dprintf("Ping COG thread %d\n", n);
}

int getPingTime(void)
{
    return PingTime;
}


/**
 * Main program function.
 */
int propBoePingTest(void)
{
    /* keep QuickStart TX quiet. */
    DIRA |= (1<<30);

    setbuf(stdout, 0);

    startPingThread();

    for(;;) {
        putchar(0);
        printf("PropBOE Ping Test\n");
        printf("\n$ ");
    }
    return 0;
}

