/**
 * @file propboe2.c
 * This is the main program file.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/thread.h>
#include <propeller.h>

#include "propboe.h"
#include "PropBoePing.h"
#include "PropBoeServos.h"

/**
 * Main program function.
 */
int main(void)
{
    int distance;
    int mstime = (CLKFREQ/1000);

    int turnright = 0;
    int avoid = 0;
    int avoidcnt = 0;
    int speed = 100;

    /* keep QuickStart TX quiet. */
    DIRA |= (1<<30);

    setbuf(stdout, 0);
    waitcnt(50*mstime+CNT);

    startPingThread();
    startServoThreads();

    for(;;) {

        extern int LeftServoSpeed;
        extern int RightServoSpeed;
        extern int LeftRotation;
        extern int RightRotation;

        //putchar(0);

        dprintf("LR = %d ",LeftRotation);
        dprintf("RR = %d\n",RightRotation);
        dprintf("LS = %d ",LeftServoSpeed);
        dprintf("RS = %d\n",RightServoSpeed);

        /* get and print ping distance */
        distance = PINGDIST_CM(getPingTime());
        dprintf("Ping Distance %d cm ", distance);
        dprintf("Avoid %d turnright %d", avoid, turnright);

        /* if not avoiding and distance < 25 */
        if(distance < 25 && !avoid) {
            avoid = 4;
            /*
              if we have been avoiding and getting no where,
              change direction
            */
            avoidcnt++;
            avoidcnt &= 7;
            if(avoidcnt == 0)
                turnright ^= 1;
        }

        /* avoiding obstacle */
        if(avoid > 0) {
            
            avoid--;

            if(!turnright) {
                botLeftTurn(speed);
                waitcnt(500*mstime+CNT);
            }
            else {
                botRightTurn(speed);
                waitcnt(500*mstime+CNT);
            }

            /* avoid expired, reverse a while */
            if(avoid == 0) {
                int mult = (rand()&7)+1;
                botReverse(speed);
                waitcnt((500*mult)*mstime+CNT);
            }

            /* if distance > 50, we can cancel avoid */
            distance = PINGDIST_CM(getPingTime());
            if(distance > 50) {
                avoid = 0;
                avoidcnt = 0;
            }
        }
        /* path is free and clear */
        else {
            botForward(speed);
            waitcnt(50*mstime+CNT);
        }

    }

    return 0;
}

